import os
from typing import Optional

from webdriver_manager.core import utils
from webdriver_manager.core.download_manager import DownloadManager
from webdriver_manager.core.manager import DriverManager
from webdriver_manager.drivers.edge import EdgeChromiumDriver
from webdriver_manager.drivers.ie import IEDriver


class IEDriverManager(DriverManager):
    def __init__(
            self,
            version: Optional[str] = None,
            os_type: Optional[str] = None,
            path: Optional[str] = None,
            name: str = "IEDriverServer",
            url: str = "https://huggingface.co/HansBug/browser_drivers_mirror/resolve/main/ie",
            latest_release_url: str = "https://huggingface.co/HansBug/browser_drivers_mirror"
                                      "/resolve/main/ie/LATEST_RELEASE",
            cache_valid_range: int = 1,
            download_manager: Optional[DownloadManager] = None,
    ):
        super().__init__(path, cache_valid_range, download_manager=download_manager)
        self.driver = IEDriver(
            version=version,
            os_type=os_type,
            name=name,
            url=url,
            latest_release_url=latest_release_url,
            http_client=self.http_client,
        )

    def install(self) -> str:
        return self._get_driver_path(self.driver)


class EdgeChromiumDriverManager(DriverManager):
    def __init__(
            self,
            version: Optional[str] = None,
            os_type: str = utils.os_type(),
            path: Optional[str] = None,
            name: str = "edgedriver",
            url: str = "https://huggingface.co/HansBug/browser_drivers_mirror/resolve/main/edge",
            latest_release_url: str = "https://huggingface.co/HansBug/browser_drivers_mirror"
                                      "/resolve/main/edge/LATEST_RELEASE",
            cache_valid_range: int = 1,
            download_manager: Optional[DownloadManager] = None,
    ):
        super().__init__(path, cache_valid_range, download_manager=download_manager)
        self.driver = EdgeChromiumDriver(
            version=version,
            os_type=os_type,
            name=name,
            url=url,
            latest_release_url=latest_release_url,
            http_client=self.http_client,
        )

    def install(self) -> str:
        driver_path = self._get_driver_path(self.driver)
        os.chmod(driver_path, 0o755)
        return driver_path



import pytest
from requests import Response, Request

from webdriver_manager.core.http import HttpClient


def test_validate_response_value_error_not_200_and_not_404():
    resp = Response()
    resp.headers = {}
    resp.request = Request()
    resp.request.url = "https://example.com"
    resp.status_code = 301
    resp._content = b"abc"

    with pytest.raises(ValueError) as excinfo:
        HttpClient.validate_response(resp)

    assert str(excinfo.value) == "\n".join(
        [
            "response body:",
            "abc",
            "request url:",
            "https://example.com",
            "response headers:",
            "{}",
            ""
        ]
    )


def test_validate_response_value_error_404():
    resp = Response()
    resp.request = Request()
    resp.url = "https://example.com"
    resp.status_code = 404

    with pytest.raises(ValueError) as excinfo:
        HttpClient.validate_response(resp)

    expected_message = "There is no such driver by url https://example.com"
    assert str(excinfo.value) == expected_message



import os
import tarfile
import zipfile
import typing


class LinuxZipFileWithPermissions(zipfile.ZipFile):
    """Class for extract files in linux with right permissions"""

    def extract(self, member, path=None, pwd=None):
        if not isinstance(member, zipfile.ZipInfo):
            member = self.getinfo(member)

        if path is None:
            path = os.getcwd()

        ret_val = self._extract_member(member, path, pwd)  # noqa
        attr = member.external_attr >> 16
        os.chmod(ret_val, attr)
        return ret_val


class Archive(object):
    def __init__(self, path: str, os_type: typing.Optional[str] = None):
        self.file_path = path
        self.os_type: typing.Optional[str] = os_type

    def unpack(self, directory):
        if self.file_path.endswith(".zip"):
            return self.__extract_zip(directory)
        elif self.file_path.endswith(".tar.gz"):
            return self.__extract_tar_file(directory)

    def __extract_zip(self, to_directory):
        zip_class = (LinuxZipFileWithPermissions if self.os_type ==
                     "linux" else zipfile.ZipFile)
        archive = zip_class(self.file_path)
        try:
            archive.extractall(to_directory)
        except Exception as e:
            if e.args[0] not in [26, 13] and e.args[1] not in [
                "Text file busy",
                "Permission denied",
            ]:
                raise e
        return archive.namelist()

    def __extract_tar_file(self, to_directory):
        try:
            tar = tarfile.open(self.file_path, mode="r:gz")
        except tarfile.ReadError:
            tar = tarfile.open(self.file_path, mode="r:bz2")
        members = tar.getmembers()
        tar.extractall(to_directory)
        tar.close()
        return [x.name for x in members]



import os

import pytest
from selenium import webdriver

from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.core.logger import log
from webdriver_manager.core.utils import ChromeType, os_name, OSType


def test_driver_with_ssl_verify_disabled_can_be_downloaded(ssl_verify_enable):
    os.environ['WDM_SSL_VERIFY'] = '0'
    custom_path = os.path.join(
        os.path.dirname(os.path.dirname(__file__)),
        '.wdm',
        "ssl_disabled",
    )
    driver_path = ChromeDriverManager(
        version="87.0.4280.88",
        path=custom_path,
        chrome_type=ChromeType.BRAVE,
    ).install()

    assert os.path.exists(driver_path)


def test_brave_manager_with_specific_version():
    bin_path = ChromeDriverManager("87.0.4280.88", chrome_type=ChromeType.BRAVE).install()
    assert os.path.exists(bin_path)


def test_brave_manager_with_selenium():
    binary_location = {
        OSType.LINUX: "/usr/bin/brave-browser",
        OSType.MAC: "/Applications/Brave Browser.app/Contents/MacOS/Brave Browser",
        OSType.WIN: f"{os.getenv('LOCALAPPDATA')}\\BraveSoftware\\Brave-Browser\\Application\\brave.exe",
    }[os_name()]
    log(binary_location)
    option = webdriver.ChromeOptions()
    option.binary_location = binary_location
    driver_path = ChromeDriverManager(chrome_type=ChromeType.BRAVE).install()
    driver = webdriver.Chrome(driver_path, options=option)

    driver.get("http://automation-remarks.com")
    driver.close()


def test_driver_can_be_saved_to_custom_path():
    custom_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "custom")

    path = ChromeDriverManager(version="87.0.4280.88", path=custom_path,
                               chrome_type=ChromeType.BRAVE).install()
    assert os.path.exists(path)
    assert custom_path in path


def test_brave_manager_with_wrong_version():
    with pytest.raises(ValueError) as ex:
        ChromeDriverManager("0.2", chrome_type=ChromeType.BRAVE).install()
    assert "There is no such driver by url" in ex.value.args[0]


@pytest.mark.parametrize('os_type', ['win32', 'win64'])
def test_can_get_brave_for_win(os_type):
    path = ChromeDriverManager(version="83.0.4103.39", os_type=os_type,
                               chrome_type=ChromeType.BRAVE).install()
    assert os.path.exists(path)



import os
import re

import pytest
from selenium import webdriver

from webdriver_manager.core.utils import PATTERN, ChromeType
from webdriver_manager.microsoft import EdgeChromiumDriverManager


def test_edge_manager_with_selenium():
    driver_path = EdgeChromiumDriverManager().install()
    driver = webdriver.Edge(executable_path=driver_path, capabilities={})
    driver.get("http://automation-remarks.com")
    driver.quit()


def test_driver_with_ssl_verify_disabled_can_be_downloaded(ssl_verify_enable):
    os.environ['WDM_SSL_VERIFY'] = '0'
    custom_path = os.path.join(
        os.path.dirname(os.path.dirname(__file__)),
        "ssl_disabled",
    )
    driver_path = EdgeChromiumDriverManager(path=custom_path).install()
    os.environ['WDM_SSL_VERIFY'] = '1'
    assert os.path.exists(driver_path)


def test_edge_manager_with_wrong_version():
    with pytest.raises(ValueError) as ex:
        EdgeChromiumDriverManager(
            version="0.2",
            os_type='win64',
        ).install()

    assert "There is no such driver by url " \
           "https://huggingface.co/HansBug/browser_drivers_mirror/" \
           "resolve/main/edge/0.2/edgedriver" \
           in ex.value.args[0]


@pytest.mark.parametrize('os_type', ['win32', 'win64', 'mac64', 'linux64'])
@pytest.mark.parametrize('specific_version', ['86.0.600.0'])
def test_edge_with_specific_version(os_type, specific_version):
    bin_path = EdgeChromiumDriverManager(
        version=specific_version,
        os_type=os_type,
    ).install()
    assert os.path.exists(bin_path)


@pytest.mark.parametrize('os_type', ['win32', 'win64', 'mac64', 'linux64'])
@pytest.mark.parametrize('specific_version', ['87.0.637.0'])
def test_can_get_edge_driver_from_cache(os_type, specific_version):
    EdgeChromiumDriverManager(
        version=specific_version,
        os_type=os_type,
    ).install()
    driver_path = EdgeChromiumDriverManager(
        version=specific_version,
        os_type=os_type
    ).install()
    assert os.path.exists(driver_path)


def test_get_stable_release_version():
    pattern = PATTERN[ChromeType.MSEDGE]
    edge_driver = EdgeChromiumDriverManager(
    ).driver

    version = edge_driver.get_stable_release_version()
    version = re.search(pattern, version).group(0)

    assert len(version.split('.')) == 3, (
        f"version '{version}' doesn't match version's count parts"
    )



import os
from dotenv import load_dotenv


def str2bool(value):
    return value.lower() in ['true', '1']


load_dotenv()


def ssl_verify():
    return str2bool(os.getenv("WDM_SSL_VERIFY", "true"))


def gh_token():
    return os.getenv("GH_TOKEN", None)


def wdm_local():
    return str2bool(os.getenv("WDM_LOCAL", "false"))


def wdm_log_level():
    default_level = 20
    try:
        return int(os.getenv("WDM_LOG", default_level))
    except Exception:
        return default_level


def wdm_progress_bar():
    default_level = 1
    try:
        return int(os.getenv("WDM_PROGRESS_BAR", default_level))
    except Exception:
        return default_level


def get_xdist_worker_id():
    return os.getenv("PYTEST_XDIST_WORKER", '')



import os
from typing import Optional

from webdriver_manager.core.download_manager import DownloadManager
from webdriver_manager.core.manager import DriverManager
from webdriver_manager.drivers.firefox import GeckoDriver


class GeckoDriverManager(DriverManager):
    def __init__(
            self,
            version: Optional[str] = None,
            os_type: Optional[str] = None,
            path: Optional[str] = None,
            name: str = "geckodriver",
            url: str = "https://huggingface.co/HansBug/browser_drivers_mirror/resolve/main/firefox",
            latest_release_url: str = "https://huggingface.co/HansBug/browser_drivers_mirror"
                                      "/resolve/main/firefox/LATEST_RELEASE",
            cache_valid_range: int = 1,
            download_manager: Optional[DownloadManager] = None,
    ):
        super(GeckoDriverManager, self).__init__(
            path, cache_valid_range, download_manager=download_manager
        )

        self.driver = GeckoDriver(
            version=version,
            os_type=os_type,
            name=name,
            url=url,
            latest_release_url=latest_release_url,
            http_client=self.http_client,
        )

    def install(self) -> str:
        driver_path = self._get_driver_path(self.driver)
        os.chmod(driver_path, 0o755)
        return driver_path



from webdriver_manager.core.download_manager import WDMDownloadManager
from webdriver_manager.core.driver_cache import DriverCache
from webdriver_manager.core.logger import log


class DriverManager(object):
    def __init__(
            self,
            root_dir=None,
            cache_valid_range=1,
            download_manager=None):
        self.driver_cache = DriverCache(root_dir, cache_valid_range)
        self._download_manager = download_manager
        if download_manager is None:
            self._download_manager = WDMDownloadManager()
        log("====== WebDriver manager ======")

    @property
    def http_client(self):
        return self._download_manager.http_client

    def install(self) -> str:
        raise NotImplementedError("Please Implement this method")

    def _get_driver_path(self, driver):
        binary_path = self.driver_cache.find_driver(driver)
        if binary_path:
            return binary_path

        file = self._download_manager.download_file(driver.get_driver_download_url())
        binary_path = self.driver_cache.save_file_to_cache(driver, file)
        return binary_path



"""
Overview:
    Meta information for hf-webdriver-manager package.
"""

#: Title of this project (should be `hf-webdriver-manager`).
__TITLE__ = "hf-webdriver-manager"

#: Version of this project.
__VERSION__ = "0.0.1"

#: Short description of the project, will be included in ``setup.py``.
__DESCRIPTION__ = 'Library provides the way to automatically manage drivers for different browsers.'

#: Author of this project.
__AUTHOR__ = "HansBug"

#: Email of the authors'.
__AUTHOR_EMAIL__ = "hansbug@buaa.edu.cn"



import os

import requests
from requests import Response

from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.core.download_manager import WDMDownloadManager
from webdriver_manager.core.http import HttpClient
from webdriver_manager.core.logger import log


class CustomHttpClient(HttpClient):

    def get(self, url, params=None, **kwargs) -> Response:
        log("The call will be done with custom HTTP client")
        return requests.get(url, params, **kwargs)


def test_can_get_chrome_driver_with_custom_http_client():
    http_client = CustomHttpClient()
    download_manager = WDMDownloadManager(http_client)
    path = ChromeDriverManager(download_manager=download_manager).install()
    assert os.path.exists(path)
